program Normaliser1;

uses
  Forms, SysUtils,
  Voxel in 'Voxel.pas',
  Voxel_Tools in 'Voxel_Tools.pas';

{$R *.RES}

type actions = (ANALYSE, APPLY);

var
   SrcFilename,
   NormalsSchemeFilename,
   DestFilename: string;
   NormalsScheme: TNormalsScheme;
   action: actions;
   src: TVoxel;

procedure PrintFacingReport(section: TVoxelSection);
var report: FacingReport;
    outfile: Text;
    normals_scheme: string;
begin
     // prepare section
     WriteLn('Analysing section "',Trim(section.Name),'"..');
     WriteLn(removeRedundantVoxels(section),' redundant voxels removed');
     WriteLn('Calculating facings now..');
     // open outfile
     AssignFile(outfile,DestFilename);
     if FileExists(DestFilename) then
        Append(outfile)
     else
         Rewrite(outfile);
     // generate report
     CalcFacings(section);
     report := GenerateFacingReport(section);
     WriteLn(outfile,'Normals Analysis of "',SrcFilename,'!',Trim(Section.Name),'"..');
     case Section.Tailer.Unknown of
          2: normals_scheme := 'TIBERIAN_SUN';
          4: normals_scheme := 'RED_ALERT2';
          else normals_scheme := 'UNKNOWN';
     end;
     WriteLn(outfile,'Normals scheme = ',Section.Tailer.Unknown,' ',normals_scheme);
     voxel_tools.PrintFacingReport(report,outfile);
     WriteLn(outfile);
     // close outfile
     CloseFile(outfile);
end;

function DetermineFiles: boolean; // false if filenames not found (app should exit)
type errors = (NONE, BAD_PARAMS, SRC_NOT_EXIST, NORMALS_NOT_EXIST, DEST_EXIST);
var error: errors;
    param: String;
begin
     error := NONE;
     // determine action
     param := Trim(UpperCase(ParamStr(1)));
     if param = 'ANALYSE' then begin
        action := ANALYSE;
        if ParamCount <> 3 then
           error := BAD_PARAMS;
     end else if param = 'APPLY' then begin
          action := APPLY;
          if ParamCount <> 4 then
             error := BAD_PARAMS;
     end else
         error := BAD_PARAMS;
     // determine files
     if error = NONE then begin
        // count params
        SrcFilename := ParamStr(2);
         if not FileExists(SrcFilename) then
            error := SRC_NOT_EXIST
         else begin
              if action = ANALYSE then
                 DestFilename := ParamStr(3)
              else begin
                   NormalsSchemeFilename := ParamStr(3);
                   DestFilename := ParamStr(4);
              end;
              if FileExists(DestFilename) then
                 error := DEST_EXIST;
              if action = APPLY then
                 if not FileExists(NormalsSchemeFilename) then
                    error := NORMALS_NOT_EXIST;
         end;
     end;
     // user message?
     if error <> NONE then begin
        Result := False; // assume bad
        case error of
             BAD_PARAMS: begin
                         WriteLn('Usage: ',ExtractFileName(ParamStr(0)),' ANALYSE [source vxl filename] [dest txt report filename]');
                         WriteLn('   or: ',ExtractFileName(ParamStr(0)),' APPLY [source vxl filename] [normal schemes file] [dest vxl filename]');
                         end;
             SRC_NOT_EXIST: WriteLn('Error: the source file "',SrcFilename,'" does not exist!');
             DEST_EXIST: WriteLn('Error: the destination file "',DestFilename,'" exists!');
             NORMALS_NOT_EXIST: WriteLn('Error: the normals scheme file "',NormalsSchemeFilename,'" does not exist!');
        end;
     end else
         Result := True;
end;

function LoadSrc(srcFilename: string): boolean; // return success
var s: integer;
begin
     // load voxel file
     WriteLn('Loading "',srcFilename,'"..');
     src := TVoxel.Create;
     src.LoadFromFile(srcFilename);
     if src.ErrorCode <> OK then begin
        WriteLn('Error loading source voxel file: ',src.ErrorMsg);
        Result := False;
        Exit;
     end;
     // list sections for info
     if Length(src.Section) = 0 then begin
        WriteLn('Error: Voxel file contains no sections; nothing to do!');
        Result := False;
        Exit;
     end;
     WriteLn('There are ',Length(src.Section),' section(s):');
     for s := 0 to Length(src.Section)-1 do
         with src.Section[s] do
              WriteLn(Header.Number,' ',Name,' (',Tailer.XSize,'x',Tailer.YSize,'x',Tailer.ZSize,')');
     // done
     Result := True;
end;

procedure ApplyNormals;
var s: integer;
begin
     for s := 0 to Pred(Length(src.Section)) do begin
         WriteLn('Autonormalising ',Trim(src.Section[s].Name),'..');
         voxel_tools.ApplyNormals(src.Section[s],NormalsScheme);
     end;
     src.SaveToFile(DestFilename);
     if src.ErrorCode <> OK then
        WriteLn('Error saving destination voxel file: ',src.ErrorMsg)
     else
         WriteLn('Autonormalised voxel written to "',DestFilename,'"');
end;

var sec: Integer;
begin
  // not sure we need this Application object?
  Application.Initialize;
  Application.Run;
  // preamble
  WriteLn('Will''s patent Voxel Normals Analyser v1, Dec 2001');
  // parse the source and destination files
  if not DetermineFiles then
     Halt(1);
  // load the source file
  if not LoadSrc(srcFilename) then
     Halt(2);
  // what to do?
  if action = APPLY then begin
     NormalsScheme := LoadNormalsScheme(NormalsSchemeFilename);
     if NormalsScheme.scheme = NOT_SET then
        Halt(3);
     WriteLn('Normals scheme "',NormalsSchemeFilename,'" by "',NormalsScheme.author,'"');
     ApplyNormals;
  end else begin // ANALYSE
    // do the analysis
    for sec := 0 to Length(src.Section)-1 do
        PrintFacingReport(src.Section[sec]);
    // done
    WriteLn('Report written to "',DestFilename,'"');
  end;
  //ReadLn;
end.
